#ifndef MARRAYS_H
#define MARRAYS_H

#ifndef MARRAY_H
#include "MArray.h"
#endif

#include <string.h>

class MArrayS : public MArray
{
private:
    UShort_t *fArray; //[fN] Array of fN chars

public:

    MArrayS()
    {
        fN = 0;
        fArray = NULL;
    }

    MArrayS(Int_t n)
    {
        fN = 0;
        fArray = NULL;
        if (n > 0)
            Set(n);
    }

    MArrayS(Int_t n, UShort_t *array)
    {
        // Create TArrayC object and initialize it with values of array.
        fN = 0;
        fArray = NULL;
        Set(n, array);
    }

    MArrayS(const MArrayS &array)
    {
        // Copy constructor.
        fArray = NULL;
        Set(array.fN, array.fArray);
    }

    Int_t GetSize() const
    {
        return fN;
    }

    MArrayS &operator=(const MArrayS &rhs)
    {
        // TArrayC assignment operator.
        if (this != &rhs)
            Set(rhs.fN, rhs.fArray);
        return *this;
    }

    virtual ~MArrayS()
    {
        // Delete TArrayC object.
        delete [] fArray;
        fArray = NULL;
    }

    void Adopt(Int_t n, UShort_t *array)
    {
        // Adopt array arr into TArrayC, i.e. don't copy arr but use it directly
        // in TArrayC. User may not delete arr, TArrayC dtor will do it.
        if (fArray)
            delete [] fArray;

        fN     = n;
        fArray = array;
    }

    void AddAt(UShort_t c, Int_t i)
    {
        // Add char c at position i. Check for out of bounds.
        fArray[i] = c;
    }

    UShort_t     At(Int_t i)
    {
        return fArray[i];
    }

    UShort_t    *GetArray() const
    {
        return fArray;
    }

    void Reset()
    {
        memset(fArray, 0, fN*sizeof(UShort_t));
    }

    void Set(Int_t n)
    {
        // Set size of this array to n chars.
        // A new array is created, the old contents copied to the new array,
        // then the old array is deleted.

        if (n < 0 || n==fN)
            return;

        UShort_t *temp = fArray;
        if (n != 0)
        {
            fArray = new UShort_t[n];
            if (n < fN)
                memcpy(fArray, temp, n*sizeof(UShort_t));
            else
            {
                memcpy(fArray, temp, fN*sizeof(UShort_t));
                memset(&fArray[fN], 0, (n-fN)*sizeof(UShort_t));
            }
        }
        else
        {
            fArray = NULL;
        }

        if (fN)
            delete [] temp;

        fN = n;
    }

    void Set(Int_t n, UShort_t *array)
    {
        // Set size of this array to n chars and set the contents.
        if (n < 0 || array == 0)
            return;

        if (fArray && fN != n)
        {
            delete [] fArray;
            fArray = 0;
        }
        fN = n;

        if (fN == 0)
            return;

        if (!fArray)
            fArray = new UShort_t[fN];

        memcpy(fArray, array, n*sizeof(UShort_t));
    }

    UShort_t &operator[](Int_t i)
    {
        return fArray[i];
    }

    ClassDef(MArrayS, 1)  //Array of UShort_t
};

#endif
